/***************************************************************************
 $RCSfile: chameleon.c,v $
                             -------------------
    cvs         : $Id: chameleon.c,v 1.5 2003/03/28 17:54:30 aquamaniac Exp $
    begin       : Wed Oct 03 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef __declspec
# if BUILDING_CHIPCARD_DLL
#  define CHIPCARD_API __declspec (dllexport)
# else /* Not BUILDING_CHIPCARD_DLL */
#  define CHIPCARD_API __declspec (dllimport)
# endif /* Not BUILDING_CHIPCARD_DLL */
#else
# define CHIPCARD_API
#endif


#include <stdio.h>
#include "chameleon/error.h"
#include "chameleon/inetaddr.h"
#include "chameleon/inetsocket.h"
#include "chameleon/ipcmessage.h"
#ifdef HAVE_OPENSSL
# include "chameleon/cryp.h"
#endif
#include "chameleon/libloader.h"
#include "chameleon/debug.h"


ERRORCODE Chameleon_Init() {
  ERRORCODE err;

  Error_ModuleInit();
#ifdef HAVE_OPENSSL
  err=Cryp_ModuleInit();
  if (!Error_IsOk(err))
    return err;
#endif
  err=InetAddr_ModuleInit();
  if (!Error_IsOk(err))
    return err;
  err=Socket_ModuleInit();
  if (!Error_IsOk(err))
    return err;
  err=IPCMessage_ModuleInit();
  if (!Error_IsOk(err))
    return err;
  err=LibLoader_ModuleInit();
  if (!Error_IsOk(err))
    return err;
  //add here more modules

  return 0;
}


ERRORCODE Chameleon_Fini() {
    ERRORCODE err;

    err=0;
    //add here more modules
    if (!Error_IsOk(LibLoader_ModuleFini())) {
      err=Error_New(0,
		    ERROR_SEVERITY_ERR,
		    0,
		    ERROR_COULD_NOT_UNREGISTER);
      DBG_ERROR("Chameleon_Fini: "
		"Could not deinitialze module LibLoader");
    }
#ifdef HAVE_SSL
    if (!Error_IsOk(Cryp_ModuleFini())) {
      err=Error_New(0,
		    ERROR_SEVERITY_ERR,
		    0,
		    ERROR_COULD_NOT_UNREGISTER);
      DBG_ERROR("Chameleon_Fini: "
		"Could not deinitialze module Cryp");
    }
#endif
    if (!Error_IsOk(IPCMessage_ModuleFini())) {
      err=Error_New(0,
		    ERROR_SEVERITY_ERR,
		    0,
		    ERROR_COULD_NOT_UNREGISTER);
      DBG_ERROR("Chameleon_Fini: "
		"Could not deinitialze module IPCMessage");
    }
    if (!Error_IsOk(Socket_ModuleFini())) {
      err=Error_New(0,
		    ERROR_SEVERITY_ERR,
		    0,
		    ERROR_COULD_NOT_UNREGISTER);
      DBG_ERROR("Chameleon_Fini: "
		"Could not deinitialze module Socket");
    }
    if (!Error_IsOk(InetAddr_ModuleFini())) {
      err=Error_New(0,
		    ERROR_SEVERITY_ERR,
		    0,
		    ERROR_COULD_NOT_UNREGISTER);
      DBG_ERROR("Chameleon_Fini: "
		"Could not deinitialze module InetAddr");
    }
    Error_ModuleFini();
    return err;
}



void Chameleon_DumpString(const char *s, int l) {
  unsigned int i;
  unsigned int j;
  unsigned int pos;

  pos=0;
  fprintf(stderr,"String size is %d:\n",l);
  while(pos<l) {
    fprintf(stderr,"%04x: ",pos);
    j=pos+16;
    if (j>=l)
      j=l;

    // show hex dump
    for (i=pos; i<j; i++) {
      fprintf(stderr,"%02x ",(unsigned char)s[i]);
    }
    if (j-pos<16)
      for (i=0; i<16-(j-pos); i++)
	fprintf(stderr,"   ");
    // show text
    for (i=pos; i<j; i++) {
      if (s[i]<32)
	fprintf(stderr,".");
      else
	fprintf(stderr,"%c",s[i]);
    }
    fprintf(stderr,"\n");
    pos+=16;
  }
}


